/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.fleet.machine.tree;

import java.awt.Image;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.util.List;
import javax.swing.AbstractAction;
import nl.cloudfarming.client.fleet.machine.windows.ListOverviewTopComponent;
import nl.cloudfarming.client.fleet.model.Machine;
import nl.cloudfarming.client.fleet.model.Machine.MachineMetaInfo;
import nl.cloudfarming.client.geoviewer.SimpleLayer;
import org.netbeans.api.project.Project;
import org.openide.filesystems.*;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.datatransfer.PasteType;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;

/**
 *
 * @author Wytse Visser
 */
public abstract class MachineCategoryNode extends FilterNode {

    private Project project;

    public MachineCategoryNode(Node original, Children children, Project project, Lookup lookup) {
        super(original, children, createLookup(original, lookup));
        this.project = project;
        setDisplayName(getMetaInfo().getDisplayName());
    }

    private static Lookup createLookup(Node original, Lookup lookup) {
        String name = lookup.lookup(MachineMetaInfo.class).getDisplayName();

        return new ProxyLookup(original.getLookup(), lookup, Lookups.fixed(new SimpleLayer(name)));
    }

    @Override
    public PasteType getDropType(Transferable t, int action, int index) {
        return null;
    }

    @Override
    public Image getIcon(int type) {
        return ImageUtilities.loadImage(getMetaInfo().getSmallIconResource());
    }

    @Override
    public Image getOpenedIcon(int type) {
        return getIcon(type);
    }

    public final MachineMetaInfo getMetaInfo() {
        return getLookup().lookup(MachineMetaInfo.class);
    }
    
    protected Project getProject() {
        return project;
    }

    @NbBundle.Messages({"fleet.machine.category_node_open_action_name=Open"})
    protected class OpenAction extends AbstractAction {

        public OpenAction() {
            super(Bundle.fleet_machine_category_node_open_action_name());
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            ListOverviewTopComponent overviewComponent = ListOverviewTopComponent.findInstance(MachineCategoryNode.this);
            overviewComponent.open();
            overviewComponent.requestActive();
        }

        @Override
        public boolean isEnabled() {
            return MachineCategoryNode.this.getChildren().getNodesCount(true) > 0;
        }
    }

}